package tests::TestHashConfigTest;

use strict;

use base qw/Test::Unit::TestCase/;

use Lire::Config;
use Lire::Test::HashConfig;

sub set_up {
    my $self = $_[0];

    $self->{cfg_struct} = { dir => '/etc/',
                            file => '/etc/passwd',
                            list_1 => [ 1..3 ],
                            hash_1 => { a => 'b',
                                        b => 'c', },
                          };
    $self->{cfg} = new Lire::Test::HashConfig( $self->{cfg_struct} );
}

sub test_new {
    my $self = $_[0];

    my $cfg = $self->{cfg};

    $self->assert_not_null( $cfg, "new() returned undef" );
    $self->assert( UNIVERSAL::isa( $cfg, "Lire::Config" ),
                   "$cfg isn't a Lire::Config object" );
}

sub test_get {
    my $self = $_[0];

    my $cfg = $self->{cfg};
    my $cfg_struct = $self->{cfg_struct};
    foreach my $var_name ( keys %$cfg_struct ) {
        my $value = $cfg->get( $var_name );
        $self->assert_not_null( $value, "get( $var_name ) returned undef" );
        $self->assert_equals( $value, $cfg_struct->{$var_name} );
    }

    eval { $cfg->get( 'unknown' ) };
    $self->assert( $@, "get() on non-existent value didn't throw an exception" );
    $self->assert( $@ =~ /No such configuration variable/,
                   "expected 'No such configuration variable' error, got '$@'"
                 );
}

sub test_Lire_Config_get {
    my $self = $_[0];

    local $Lire::Config::SINGLETON = $self->{cfg};

    my $cfg_struct = $self->{cfg_struct};
    foreach my $var_name ( keys %$cfg_struct ) {
        my $value = Lire::Config->get( $var_name );
        $self->assert_not_null( $value, "get( $var_name ) returned undef" );
        $self->assert_equals( $value, $cfg_struct->{$var_name} );
    }
}

sub test_unimplemented_methods {
    my $self = $_[0];

    my $cfg = $self->{cfg};
    foreach my $method ( qw/get_var config_spec config_spec_path 
                            add_config_spec_path_dir del_config_spec_path_dir
                            config_files add_config_path new_config_file
                            add_config_file del_config_file get_config_file / )
    {
        no strict 'refs';
        eval { $cfg->$method() };
        $self->assert( $@, "method $method() didn't failed" );
        $self->assert( $@ =~ /Unimplemented method/,
                       "expected 'Unimplemented method' error, got '$@'" );
    }
}

1;
